@@ -191,7 +191,12 @@ local function prepare_dkim_signing(N, task, settings)
   elseif settings.sign_inbound and not is_local and not auser then
     lua_util.debugm(N, task, 'mail was sent to us')
   else
-    lua_util.debugm(N, task, 'mail is ineligible for signing')
+    lua_util.debugm(N, task,
+      'mail is ineligible for signing: auser=%s, ip=%s, is_local=%s, ' ..
+      'sign_authenticated=%s, sign_networks=%s, sign_local=%s, sign_inbound=%s',
+      auser, tostring(ip), is_local,
+      settings.sign_authenticated, settings.sign_networks ~= nil,
+      settings.sign_local, settings.sign_inbound)
     return false, {}
   end
 
@@ -617,8 +622,22 @@ exports.sign_using_vault = function(N, task, settings, selector, sign_func, err_
   local http = require "rspamd_http"
   local ucl = require "ucl"
 
+  local vault_path = settings.vault_path or 'dkim'
+  local vault_kv_version = settings.vault_kv_version or 1
+
+  -- For KV v2, we need to add 'data' to the path for read operations
+  if vault_kv_version == 2 then
+    local mount_point = vault_path:match('^([^/]+)')
+    local subpath = vault_path:match('^[^/]+/?(.*)')
+    if subpath and subpath ~= '' then
+      vault_path = mount_point .. '/data/' .. subpath
+    else
+      vault_path = mount_point .. '/data'
+    end
+  end
+
   local full_url = string.format('%s/v1/%s/%s',
-    settings.vault_url, settings.vault_path or 'dkim', selector.domain)
+    settings.vault_url, vault_path, selector.domain)
   local upstream_list = lua_util.http_upstreams_by_url(rspamd_config:get_mempool(), settings.vault_url)
 
   local function vault_callback(err, code, body, _)
@@ -638,7 +657,10 @@ exports.sign_using_vault = function(N, task, settings, selector, sign_func, err_
           err_func(task, string.format('vault reply for %s (data=%s) is invalid, no data',
             full_url, body))
         else
-          local elts = obj.data.selectors or {}
+          -- For KV v2, data is nested under obj.data.data
+          -- For KV v1, data is under obj.data
+          local vault_data = vault_kv_version == 2 and obj.data.data or obj.data
+          local elts = vault_data and vault_data.selectors or {}
           local errs = {}
           local nvalid = 0
 
