@@ -261,6 +261,21 @@ define(["jquery", "app/common", "footable"],
                 $create: function () {
                     this._super();
                     const self = this;
+
+                    if (self.$input && self.$input.length && !self.$input.parent().find(".search-syntax-icon").length) {
+                        self.$input.parent().css("position", "relative");
+                        const $icon = $("<i/>", {
+                            class: "fas fa-circle-question search-syntax-icon text-muted",
+                            title: "Search syntax: match all rows containing\n\n" +
+                                   "\"exact phrase\" — exact string (including spaces)\n" +
+                                   "term1 OR term2 — either term\n" +
+                                   "term1 AND term2 — both terms\n" +
+                                   "term1 term2 — both terms (same as AND)\n" +
+                                   "term1 -term2 — term1 but exclude rows with term2"
+                        });
+                        $icon.insertAfter(self.$input);
+                    }
+
                     const $form_grp = $("<div/>", {
                         class: "form-group d-inline-flex align-items-center"
                     }).append($("<label/>", {
@@ -318,6 +333,13 @@ define(["jquery", "app/common", "footable"],
                         self.removeFilter("action");
                     }
                     self.filter();
+                },
+                draw: function () {
+                    // Ensure the dropdown reflects the default value when filters are cleared.
+                    this._super();
+                    const actionFilter = this.find("action");
+                    const isActionFilterApplied = actionFilter instanceof FooTable.Filter;
+                    if (this.$action && !isActionFilterApplied) this.$action.val(this.def);
                 }
             });
             /* eslint-enable consistent-this, no-underscore-dangle */
@@ -532,7 +554,7 @@ define(["jquery", "app/common", "footable"],
             const unsorted_symbols = [];
             const compare_function = get_compare_function(table);
 
-            $("#selSymOrder_" + table + ", label[for='selSymOrder_" + table + "']").show();
+            common.show("#selSymOrder_" + table + ", label[for='selSymOrder_" + table + "]");
 
             $.each(data.rows,
                 (i, item) => {
@@ -573,17 +595,15 @@ define(["jquery", "app/common", "footable"],
 
                     ui.preprocess_item(item);
                     Object.values(item.symbols).forEach((sym) => {
-                        sym.str = '<span class="symbol-default ' + get_symbol_class(sym.name, sym.score) + '"><strong>';
-
-                        if (sym.description) {
-                            sym.str += '<abbr title="' + sym.description + '">' + sym.name + "</abbr>";
-                        } else {
-                            sym.str += sym.name;
-                        }
-                        sym.str += "</strong> (" + sym.score + ")</span>";
+                        sym.str = `
+<span class="symbol-default ${get_symbol_class(sym.name, sym.score)} ${sym.description ? "has-description" : ""}" tabindex="0">
+    <strong>${sym.name}</strong>
+    ${sym.description ? `<span class="symbol-description"> • ${sym.description}</span>` : ""}
+    (${sym.score})
+</span>`;
 
                         if (sym.options) {
-                            sym.str += " [" + sym.options.join(",") + "]";
+                            sym.str += ` [${sym.options.join(",")}]`;
                         }
                     });
                     unsorted_symbols.push(item.symbols);
