@@ -139,9 +139,9 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                         () => module.statWidgets(graphs, checked_server));
                     if (id !== "#autoRefresh") module.statWidgets(graphs, checked_server);
 
-                    $(".preset").show();
-                    $(".history").hide();
-                    $(".dynamic").hide();
+                    common.show(".preset");
+                    common.hide(".history");
+                    common.hide(".dynamic");
                 });
                 break;
             case "#throughput_nav":
@@ -161,9 +161,9 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                         () => module.draw(graphs, common.neighbours, checked_server, selData));
                     if (id !== "#autoRefresh") module.draw(graphs, common.neighbours, checked_server, selData);
 
-                    $(".preset").hide();
-                    $(".history").hide();
-                    $(".dynamic").show();
+                    common.hide(".preset");
+                    common.hide(".history");
+                    common.show(".dynamic");
                 });
                 break;
             case "#configuration_nav":
@@ -195,9 +195,9 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                         () => getHistoryAndErrors());
                     if (id !== "#autoRefresh") getHistoryAndErrors();
 
-                    $(".preset").hide();
-                    $(".history").show();
-                    $(".dynamic").hide();
+                    common.hide(".preset");
+                    common.show(".history");
+                    common.hide(".dynamic");
 
                     module.updateHistoryControlsState();
                 });
@@ -245,15 +245,20 @@ define(["jquery", "app/common", "stickytabs", "visibility",
 
                 if (common.read_only) {
                     $(".ro-disable").attr("disabled", true);
-                    $(".ro-hide").hide();
+                    common.hide(".ro-hide");
                 } else {
                     $(".ro-disable").removeAttr("disabled", true);
-                    $(".ro-hide").show();
+                    common.show(".ro-hide");
                 }
 
                 $("#preloader").addClass("d-none");
                 $("#navBar, #mainUI").removeClass("d-none");
-                $(".nav-tabs-sticky").stickyTabs({initialTab: "#status_nav"});
+
+                // Initialize FontAwesome icon replacement for FooTable before activating tabs
+                require(["app/footable-fontawesome"], (FootableFA) => {
+                    FootableFA.init();
+                    $(".nav-tabs-sticky").stickyTabs({initialTab: "#status_nav"});
+                });
             },
             errorMessage: "Cannot get server status",
             server: "All SERVERS"
@@ -280,7 +285,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
             error: function () {
                 function clearFeedback() {
                     $("#connectPassword").off("input").removeClass("is-invalid");
-                    $("#authInvalidCharFeedback,#authUnauthorizedFeedback").hide();
+                    common.hide("#authInvalidCharFeedback,#authUnauthorizedFeedback");
                 }
 
                 $("#connectDialog")
@@ -302,7 +307,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                         $("#connectPassword")
                             .addClass("is-invalid")
                             .off("input").on("input", () => clearFeedback());
-                        $(tooltip).show();
+                        common.show(tooltip);
                     }
 
                     if (!(/^[\u0020-\u007e]*$/).test(password)) {
@@ -330,7 +335,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                             if (textStatus.statusText === "Unauthorized") {
                                 invalidFeedback("#authUnauthorizedFeedback");
                             } else {
-                                common.alertMessage("alert-modal alert-error", textStatus.statusText);
+                                common.alertMessage("alert-modal alert-danger", textStatus.statusText);
                             }
                             $("#connectPassword").val("");
                             $("#connectPassword").focus();
@@ -345,6 +350,22 @@ define(["jquery", "app/common", "stickytabs", "visibility",
         });
     };
 
+    function updateThemeIcon(theme) {
+        const icon = $("#theme-icon");
+        icon.removeClass("fa-moon fa-sun fa-display");
+
+        switch (theme) {
+            case "light":
+                icon.addClass("fa-sun");
+                break;
+            case "dark":
+                icon.addClass("fa-moon");
+                break;
+            default:
+                icon.addClass("fa-display");
+                break;
+        }
+    }
 
     (function initSettings() {
         let selected_locale = null;
@@ -412,6 +433,10 @@ define(["jquery", "app/common", "stickytabs", "visibility",
             ajaxSetup(localStorage.getItem("ajax_timeout"), true);
 
             $(historyCountSelector).val(parseInt(localStorage.getItem("historyCount"), 10) || historyCountDef);
+
+            // Restore theme selection
+            const savedTheme = localStorage.getItem("theme") || "auto";
+            $('.popover #settings-popover input:radio[name="theme"]').val([savedTheme]);
         });
         $(document).on("change", '.popover #settings-popover input:radio[name="locale"]', function () {
             selected_locale = this.value;
@@ -422,6 +447,14 @@ define(["jquery", "app/common", "stickytabs", "visibility",
             custom_locale = $(localeTextbox).val();
             validateLocale(true);
         });
+        $(document).on("change", '.popover #settings-popover input:radio[name="theme"]', function () {
+            const theme = this.value;
+            if (window.rspamd && window.rspamd.theme) {
+                window.rspamd.theme.applyPreference(theme);
+            }
+            updateThemeIcon(theme || "auto");
+        });
+        updateThemeIcon(localStorage.getItem("theme") || "auto");
         $(document).on("input", ajaxTimeoutBox, () => {
             ajaxSetup($(ajaxTimeoutBox).val(), false, true);
         });
@@ -487,6 +520,34 @@ define(["jquery", "app/common", "stickytabs", "visibility",
         tabClick("#autoRefresh");
     });
 
+    $("#theme-toggle").on("click", (e) => {
+        e.preventDefault();
+        const currentTheme = localStorage.getItem("theme") || "auto";
+        // eslint-disable-next-line no-useless-assignment
+        let newTheme = null;
+
+        // Cycle through: light -> dark -> auto -> light
+        switch (currentTheme) {
+            case "light":
+                newTheme = "dark";
+                break;
+            case "dark":
+                newTheme = "auto";
+                break;
+            default:
+                newTheme = "light";
+                break;
+        }
+
+        if (window.rspamd && window.rspamd.theme) {
+            window.rspamd.theme.applyPreference(newTheme);
+        }
+        updateThemeIcon(newTheme);
+
+        // Update radio button in settings popover if it's open
+        $('.popover #settings-popover input:radio[name="theme"]').val([newTheme]);
+    });
+
     $("#selSrv").change(function () {
         checked_server = this.value;
         $("#selSrv [value=\"" + checked_server + "\"]").prop("checked", true);
